/**
 * This file demonstrates the group lookup capabilities of the API. The lookup
 * allows you to create groups of modules based on which ones are available on
 * the network.
 */

#include <stdio.h>
#include "hebi_lookup.h"
#include "hebi_mac_address.h"
#include "hebi_info.h"
#include "hebi_util.h"

/**
 * Helper function to create mac address
 */
HebiMacAddress hebiMacAddressFromBytes(
  uint8_t a, uint8_t b, uint8_t c, uint8_t d, uint8_t e, uint8_t f)
{
  HebiMacAddress tmp;
  tmp.bytes_[0] = a;
  tmp.bytes_[1] = b;
  tmp.bytes_[2] = c;
  tmp.bytes_[3] = d;
  tmp.bytes_[4] = e;
  tmp.bytes_[5] = f;
  return tmp;
}

/**
 * Simple helper function to print out if the group is valid, and release any
 * reference to it.
 */
void checkAndReleaseGroup(HebiGroupPtr group)
{
  if (group == NULL)
  {
    printf("Group not found on network.\n\n");
    return;
  }

  printf("Found group with %d modules.\n\n", hebiGroupGetNumberOfModules(group));

  // Be sure the group is released after you are done with it.
  hebiReleaseGroup(group);
}

/**
 * Attempts to look for several groups on the network (hardcoded addresses below),
 * and retrieve references to them.
 */
int main()
{
  // Create the lookup object
  HebiLookupPtr lookup = hebiCreateLookup();
  if (lookup == NULL)
  {
    printf("createLookup() returned NULL.\n\n");
    hebiCleanup();
    return 1;
  }

  // Wait for the module list to populate, and print out its contents
  hebi_sleep_s(2);
  hebiPrintLookupTable(lookup);
  printf("\n");

  // Define module names/addresses:
  HebiMacAddress macs[2];
  int num_macs = sizeof(macs) / sizeof(macs[0]);
  macs[0] = hebiMacAddressFromBytes(0x00, 0x1e, 0xc0, 0x8d, 0xe0, 0x5c);
  macs[1] = hebiMacAddressFromBytes(0x00, 0x1e, 0xc0, 0x8d, 0x79, 0xd8);

  char* names[2];
  int num_names = sizeof(names) / sizeof(names[0]);
  char name_1[6] = "SA034";
  names[0] = name_1;
  char name_2[6] = "SA030";
  names[1] = name_2;

  char* families[1];
  const int num_families = sizeof(families) / sizeof(families[0]);
  char fam_name_1[4] = "arm";
  families[0] = fam_name_1;

  // Test out lookup methods; use a 4 second timeout.
  long timeout_ms = 4000;

  printf("Looking up group by MAC addresses.\n");
  HebiGroupPtr group = hebiGetGroupFromMacs(lookup, macs, num_macs, timeout_ms);
  checkAndReleaseGroup(group);

  printf("Looking up group by name.\n");
  group = hebiGetGroupFromNames(lookup,
                                (const char * const *)names, num_names,
                                (const char * const *)families, num_families, timeout_ms);
  checkAndReleaseGroup(group);

  printf("Looking up group by family.\n");
  group = hebiGetGroupFromFamily(lookup, families[0], timeout_ms);
  checkAndReleaseGroup(group);

  printf("Looking up connected group by mac.\n");
  group = hebiGetConnectedGroupFromMac(lookup, &macs[0], timeout_ms);
  checkAndReleaseGroup(group);

  printf("Looking up connected group by name.\n");
  group = hebiGetConnectedGroupFromName(lookup, names[0], families[0], timeout_ms);
  checkAndReleaseGroup(group);

  // Always delete the lookup after you are done with it.
  hebiDeleteLookup(lookup);
  hebiCleanup();

  return 0;
}
